#include <stdio.h>
#include <ctype.h>
#include <string.h>

#undef ATOMINFO_C__
#include "atominfo.h"
#include "sasaki.h"


const T_Ef1f2_Sasaki *FindEf1f2_Sasaki(const char *Label, int Exact)
{
  int                 i, m;
  char                buf[5];
  const T_Ef1f2_Sasaki  *Ef1f2Sasaki, *mEf1f2Sasaki;


  AtomInfoStripLabel(Label, Exact, buf, sizeof buf / sizeof (*buf));

  if (buf[0] == '\0')
    return NULL;

  /* map D to H */
  if (strcmp(buf, "D") == 0) buf[0] = 'H';

  m = 0;
  mEf1f2Sasaki = NULL;

  for (Ef1f2Sasaki = Ef1f2_SasakiList; Ef1f2Sasaki->Symbol; Ef1f2Sasaki++)
  {
    for (i = 0; buf[i] && Ef1f2Sasaki->Symbol[i]; i++)
      if (buf[i] != toupper(Ef1f2Sasaki->Symbol[i])) break;

    if (buf[i] == '\0' && Ef1f2Sasaki->Symbol[i] == '\0')
      return Ef1f2Sasaki;

    if (i == 1 && isalpha(Ef1f2Sasaki->Symbol[1]))
      i = 0;

    if (i > m && ! isdigit(Ef1f2Sasaki->Symbol[i - 1]))
    {
      m    = i;
      mEf1f2Sasaki = Ef1f2Sasaki;
    }
  }

  if (Exact)
    return NULL;

  return mEf1f2Sasaki;
}


/* Get f1 f2 for a given energy. Assumes Energies are increasing in the table*/
void Get_fpfs_Sasaki(const char *Label, const float Energy,
                    float *fprime, float *fsecond)
{
   const T_Ef1f2_Sasaki  *Ef1f2Sasaki;
   float Energy1, Energy2;
   int i;

   *fprime=-9999;
   *fsecond=-9999;

   Ef1f2Sasaki=FindEf1f2_Sasaki(Label,0);

   if(Ef1f2Sasaki)
   {
      Energy1=Ef1f2Sasaki->Ef1f2List[0].Energy ;
      Energy2=Ef1f2Sasaki->Ef1f2List[1].Energy ;
      for(i=2 ; (Energy2>0.1) && (Energy2<Energy) ; i++)
      {
         Energy1=Energy2;
         Energy2=Ef1f2Sasaki->Ef1f2List[i].Energy ;
      }
      if(Energy2>0)
      {
         *fprime=(Ef1f2Sasaki->Ef1f2List[i-2].f1)
                +(Ef1f2Sasaki->Ef1f2List[i-1].f1-Ef1f2Sasaki->Ef1f2List[i-2].f1)
                *(Energy-Energy1)/(Energy2-Energy1);
         *fsecond=(Ef1f2Sasaki->Ef1f2List[i-2].f2)
                 +(Ef1f2Sasaki->Ef1f2List[i-1].f2-Ef1f2Sasaki->Ef1f2List[i-2].f2)
                 *(Energy-Energy1)/(Energy2-Energy1);
      }
   }
}
